@@ -5,7 +5,7 @@
 modname = 'mercurial_server'
 distname = 'cubicweb-mercurial-server'
 
-numversion = (0, 5, 0)
+numversion = (0, 6, 0)
 version = '.'.join(str(num) for num in numversion)
 
 license = 'LGPL'
@@ -15,7 +15,9 @@ description = 'mercurial server integration cube'
 web = 'http://www.cubicweb.org/project/%s' % distname
 
 __depends__ =  {'cubicweb': '>= 3.16',
-                'cubicweb-vcsfile': '>= 1.12.2'}
+                'cubicweb-vcsfile': '>= 2',
+                'python-hglib': '>=1.3',
+                }
 __recommends__ = {}
 
 
