@@ -5,7 +5,7 @@ from contextlib import contextmanager
 from urllib import quote_plus
 from urlparse import urlparse
 
-from mercurial import commands, hg
+import hglib
 
 from logilab.common.shellutils import tempdir
 from logilab.common.decorators import monkeypatch, cachedproperty
@@ -16,8 +16,6 @@ from cubicweb import ValidationError
 from cubes.vcsfile.hooks import clone_to_local_cache
 from cubes.vcsfile.entities import Repository
 
-from cubes.mercurial_server.utils import mscui
-
 
 class CWUser(authobjs.CWUser):
 
@@ -77,14 +75,19 @@ class HGSRepository(Repository):
             clone_to_local_cache(self)
         commitopts.update(message=message)
         with fs_lock(self._cw, self.fs_repo_lock_path):
-            repo = hg.repository(mscui(), str(self.localcachepath))
-            commands.pull(repo.ui, repo,
-                          source=self.source_url.encode('ascii'))
-            commands.update(repo.ui, repo)
+            repo = hglib.open(str(self.localcachepath))
+            repo.pull(source=self.source_url.encode('ascii'))
+            repo.update()
             yield repo
-            commands.commit(repo.ui, repo, **commitopts)
-            commands.push(repo.ui, repo,
-                          dest=self.source_url.encode('ascii'))
+            try:
+                repo.commit(**commitopts)
+            except hglib.error.CommandError as exc:
+                if exc.ret == 1 and exc.out.strip() == 'nothing changed':
+                    # do not fail if nothing had to be commited
+                    return
+                raise
+            else:
+                repo.push(dest=self.source_url.encode('ascii'))
 
     @property
     def push_url(self):
@@ -122,11 +125,11 @@ class MercurialServerConfig(AnyEntity):
         with self.hgadmin_repo.pull_push(message, **commitopts) as repo:
             yield repo
 
-    def write_access_conf(self, ui, repo, access_line):
+    def write_access_conf(self, repo, access_line):
         access_conf_path = self.access_conf_path(repo)
         with open(access_conf_path, 'a') as access_file:
             access_file.write(access_line)
-        commands.add(ui, repo, access_conf_path)
+        repo.add(access_conf_path)
 
     def generate_access_conf(self):
         """ Generate the contents of the access.conf file """
