@@ -22,15 +22,13 @@ import os.path as osp
 import shutil
 from collections import defaultdict
 
-from mercurial import commands, hg, error
+import hglib
 
 from cubicweb import ValidationError
 from cubicweb.predicates import is_instance, match_user_groups
 from cubicweb.server import Service
 from cubicweb.server.hook import Hook, Operation, DataOperationMixIn, match_rtype
 
-from cubes.mercurial_server.utils import mscui
-
 # MercurialServerConfig
 
 class MSCCreateHook(Hook):
@@ -97,19 +95,17 @@ class MercurialServerRepositoryCreation(Hook):
         except UnicodeEncodeError:
             msg = self._cw._('source_url must be ASCII encodable')
             raise ValidationError(self.eidfrom, {'source_url': msg})
-        hgui = mscui()
+
         try:
-            commands.init(hgui, uri)
+            hglib.init(uri, ssh=self._cw.vreg.config['hg-ssh'] or None)
             self.warning('init-ed repository %s', uri)
-        except error.RepoError:
-            try:
-                commands.identify(hgui, None, source=uri)
-                self.warning('repo %s seems to be already there: cannot init', uri)
-            except error.RepoError:
-                raise ValidationError(self.eidfrom, {'source_url': hgui.popbuffer()})
-        else:
             # prepare possible rollback (destroy remote repository)
             MSRepoRollbackOnInit.get_instance(self._cw).add_data(uri)
+        except hglib.error.CommandError as exc:
+            if re.match(r'^mercurial-server: .+ exists$', exc.err.splitlines()[0]):
+                self.warning('repo %s seems to be already there: cannot init', uri)
+            else:
+                raise ValidationError(self.eidfrom, {'source_url': exc.err})
         # let's create a hg server permission right away
         self._cw.create_entity('MercurialServerPermission',
                                access_key=user.public_key,
@@ -143,23 +139,24 @@ class MSCDeletion(Hook):
 
     def __call__(self):
         msc = self.entity
-        hgrepo = hg.repository(mscui(), msc.hgadmin_repopath)
+        hgrepo = hglib.open(msc.hgadmin_repopath)
         try:
-            with open(osp.join(hgrepo.root, msc.conf_filename), 'wb') as conf_file:
+            with open(osp.join(hgrepo.root(), msc.conf_filename), 'wb') as conf_file:
                 conf_file.write('\n')
         except OSError, err:
             self.error('critical filesystem operation failure')
             msg = _('could not alter the hgadmin repository content (%s)' % err)
             raise ValidationError(msc.eid, {'hgadmin_repository': msg})
         try:
-            commands.remove(hgrepo.ui, hgrepo, osp.join(hgrepo.root, 'keys', msc.keys_subdir))
+            hgrepo.remove(osp.join(hgrepo.root(), 'keys', msc.keys_subdir))
             msctitle = msc.dc_title().encode('utf-8')
             message = ('server config `%s` (eid %s) deletion' % (msctitle, msc.eid))
-            commands.commit(hgrepo.ui, hgrepo, message=message)
-            cset = hgrepo[None].p1().hex()
-            commands.push(hgrepo.ui, hgrepo)
-            MSCRollbackOnDelete.get_instance(self._cw).add_data((msc.eid, msctitle,
-                                                                 hgrepo.root, cset))
+            hgrepo.commit(message=message)
+            cset = hgrepo.parents()[0][1]
+            hgrepo.push(dest=msc.hgadmin_repo.source_url, rev=cset)
+            rbdata = (msc.eid, msc.hgadmin_repo.source_url, msctitle,
+                      hgrepo.root(), cset)
+            MSCRollbackOnDelete.get_instance(self._cw).add_data(rbdata)
         except Exception, exc:
             self.error('error while handling the hgadmin repository')
             raise ValidationError(msc.eid, {'hgadmin_repo': _('error: %s') % exc})
@@ -171,22 +168,23 @@ class MSCRollbackOnDelete(DataOperationMixIn, Operation):
     """
 
     def rollback_event(self):
-        for msceid, msctitle, reporoot, cset in self.get_data():
-            hgrepo = hg.repository(mscui(), reporoot)
-            commands.backout(hgrepo.ui, hgrepo, rev=cset,
-                             message='backout `%s` (eid %s) deletion' % (msctitle, msceid))
-            commands.push(hgrepo.ui, hgrepo)
+        for msceid, pushurl, msctitle, reporoot, cset in self.get_data():
+            hgrepo = hglib.open(reporoot)
+            hgrepo.backout(rev=cset,
+                           message='backout `%s` (eid %s) deletion' % (msctitle, msceid))
+            cset = hgrepo.parents()[0][1]
+            hgrepo.push(dest=pushurl, rev=cset)
 
 
 # SshPubKey
 
-def schedule_key_sync(session, event, keyeid):
-    key = session.entity_from_eid(keyeid)
+def schedule_key_sync(cnx, event, keyeid):
+    key = cnx.entity_from_eid(keyeid)
     if not key.reverse_public_key and event == 'delete':
         # user has been deleted
         return
     for conf in key.mercurial_server_configs:
-        SynchronizeAccesKeysOperation.get_instance(session).add_data(
+        SynchronizeAccesKeysOperation.get_instance(cnx).add_data(
             (event, conf.eid, osp.join('keys', conf.keys_subdir, *key.pathtuple),
              key.data.getvalue()))
 
@@ -203,11 +201,11 @@ class CWUSerDeletion(Hook):
 class CWUserDeleteOp(DataOperationMixIn, Operation):
 
     def postcommit_event(self):
-        regenerator = self.session.vreg['services'].select('regenerate_hgadmin_repo', self.session)
-        with self.session.repo.internal_session() as session:
-            for msceid, in session.execute('MercurialServerConfig M').rows:
+        regenerator = self.cnx.vreg['services'].select('regenerate_hgadmin_repo', self.cnx)
+        with self.cnx.repo.internal_cnx() as cnx:
+            for msceid, in cnx.execute('MercurialServerConfig M').rows:
                 regenerator.call(msceid)
-            session.commit()
+            cnx.commit()
 
 class SshPubKeyUpdate(Hook):
     __regid__ = 'mercurialserver.accesskeyupdatehook'
@@ -243,9 +241,8 @@ class SshPubKeyLinkUnlink(Hook):
 class NewAccessKey(DataOperationMixIn, Operation):
 
     def precommit_event(self):
-        session = self.session
         for event, keyeid in self.get_data():
-            schedule_key_sync(self.session, event, keyeid)
+            schedule_key_sync(self.cnx, event, keyeid)
 
 
 # Permission
@@ -269,32 +266,32 @@ class MercurialServerPermission(Hook):
 
 # Operations
 
-def remove_key(session, ui, repo, keypath):
-    filename = osp.join(repo.root, keypath)
-    session.warning('remove public key %s', filename)
-    commands.remove(ui, repo, filename)
+def remove_key(cnx, repo, keypath):
+    filename = osp.join(repo.root(), keypath)
+    cnx.warning('remove public key %s', filename)
+    repo.remove(filename)
 
-def add_key(session, ui, repo, keypath, pubkey):
-    filename = osp.join(repo.root, keypath)
+def add_key(cnx, repo, keypath, pubkey):
+    filename = osp.join(repo.root(), keypath)
     dirpath = osp.dirname(filename)
     if not osp.isdir(dirpath):
         os.makedirs(dirpath)
     with open(filename, 'wb') as keyfile:
         keyfile.write(pubkey)
-        session.warning('add public key %s', filename)
-        commands.add(ui, repo, str(filename))
+        cnx.warning('add public key %s', filename)
+        repo.add(str(filename))
 
 
-def sync_access_keys(session, server_config, op_path_pubkey):
+def sync_access_keys(cnx, server_config, op_path_pubkey):
     msg = ('Access keys sync from cube mercurial-server for %s' %
            ', '.join([x[1] for x in op_path_pubkey]))
-    userdesc = session.user.format_name_for_mercurial()
+    userdesc = cnx.user.format_name_for_mercurial()
     with server_config.exec_in_hgadmin_clone(msg, user=userdesc) as repo:
         for op, path, pubkey in op_path_pubkey:
             if op == 'add':
-                add_key(session, repo.ui, repo, path, pubkey)
+                add_key(cnx, repo, path, pubkey)
             else:
-                remove_key(session, repo.ui, repo, path)
+                remove_key(cnx, repo, path)
 
 
 class SynchronizeAccesKeysOperation(DataOperationMixIn, Operation):
@@ -313,32 +310,32 @@ class SynchronizeAccesKeysOperation(DataOperationMixIn, Operation):
             by_config[config_eid].append((operation, path, pubkey))
 
         for configeid, op_path_pubkey in by_config.iteritems():
-            RegenerateAccesConfOperation.get_instance(self.session).add_data(config_eid)
-            server_config = self.session.entity_from_eid(configeid)
-            sync_access_keys(self.session, server_config, op_path_pubkey)
+            RegenerateAccesConfOperation.get_instance(self.cnx).add_data(config_eid)
+            server_config = self.cnx.entity_from_eid(configeid)
+            sync_access_keys(self.cnx, server_config, op_path_pubkey)
 
-def write_access_conf(session, msc):
+def write_access_conf(cnx, msc):
     msg = ('Regenerate access configuration %s from cube mercurial-server' %
            msc.dc_title())
     with msc.exec_in_hgadmin_clone(msg) as repo:
         access_conf_contents = msc.generate_access_conf()
-        filename = osp.join(repo.root, msc.conf_filename_str)
+        filename = osp.join(repo.root(), msc.conf_filename_str)
         already_exists = osp.exists(filename)
         with open(filename, 'w') as fobj:
             fobj.write(access_conf_contents)
         if not already_exists:
-            session.warning('write %s', filename)
-            commands.add(repo.ui, repo, filename)
+            cnx.warning('write %s', filename)
+            repo.add(filename)
 
 class RegenerateAccesConfOperation(DataOperationMixIn, Operation):
     """ Regenerate the mercurial-server <access.conf> file"""
 
     def precommit_event(self):
         for configeid in self.get_data():
-            if self.session.deleted_in_transaction(configeid):
+            if self.cnx.deleted_in_transaction(configeid):
                 continue
-            server_config = self.session.entity_from_eid(configeid)
-            write_access_conf(self.session, server_config)
+            server_config = self.cnx.entity_from_eid(configeid)
+            write_access_conf(self.cnx, server_config)
 
 
 
@@ -353,9 +350,9 @@ class CheckAsciiLoginFromRtype(Hook):
         validate_ascii_login(self._cw, self.eidto, self.eidfrom)
 
 
-def validate_ascii_login(session, msaeid, usereid):
-    login = session.execute('Any L WHERE U login L, U eid %(u)s',
-                            {'u': usereid}).rows[0][0]
+def validate_ascii_login(cnx, msaeid, usereid):
+    login = cnx.execute('Any L WHERE U login L, U eid %(u)s',
+                        {'u': usereid}).rows[0][0]
     try:
         login.encode('ascii')
     except UnicodeEncodeError:
@@ -373,19 +370,18 @@ class RegenerateMSCRepoService(Service):
         msg = "Regenerate cfg and keys"
         userdesc = self._cw.user.format_name_for_mercurial()
         with msc.exec_in_hgadmin_clone(msg, user=userdesc) as hgrepo:
-            commands.remove(hgrepo.ui, hgrepo,
-                            osp.join(hgrepo.root, 'keys', msc.keys_subdir))
+            hgrepo.remove(osp.join(hgrepo.root(), 'keys', msc.keys_subdir))
             RegenerateAccesConfOperation.get_instance(self._cw).add_data(eid)
             rset = self._cw.execute('Any K WHERE K is SshPubKey, P access_key K, '
                                     'P granted_on R, R hosted_by M, M eid %(eid)s',
                                     {'eid':eid})
             for key in rset.entities():
-                filename = osp.join(hgrepo.root, 'keys', msc.keys_subdir, *key.pathtuple)
+                filename = osp.join(hgrepo.root(), 'keys', msc.keys_subdir, *key.pathtuple)
                 dirpath = osp.dirname(filename)
                 if not osp.isdir(dirpath):
                     os.makedirs(dirpath)
                 with open(filename, 'wb') as keyfile:
                     keyfile.write(key.data.getvalue())
-                commands.add(hgrepo.ui, hgrepo, str(filename))
+                hgrepo.add(str(filename))
         # ... just in case, we don't really know who is responsible for this
         self._cw.commit()
