@@ -20,4 +20,11 @@ options = (
       'group': 'mercurial_server',
       'level': 2,
       }),
+    ('hg-ssh',
+     {'type': 'string',
+      'default': None,
+      'help': 'ssh argument passed to hglib commands',
+      'group': 'mercurial_server',
+      'level': 2,
+      }),
     )
